//    Copyright (C) 2009 Dirk Vanden Boer <dirk.vdb@gmail.com>
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef GEJENGEL_H
#define GEJENGEL_H

#include "albumartprovider.h"
#include "gejengelcore.h"
#include "playback.h"
#include "playqueue.h"
#include "settings.h"
#include "ui/mainwindow.h"
#include "musiclibrary.h"
#include "pluginmanager.h"
#include "Utils/condition.h"
#include "Utils/mutex.h"
#include "Utils/types.h"

namespace Gejengel
{

class PlaybackEngine;
class TrackSubscriber;

class GejengelCore  : public IGejengelCore
                    , public IAlbumArtProvider
                    , public PlayQueueSubscriber
{
public:
    GejengelCore();
    ~GejengelCore();

    void run();

    void play();
    void pause();
    void resume();
    void prev();
    void next();
    void stop();
    PlaybackState getPlaybackState();

    void seek(double seconds);
    double getTrackPosition();

    void setVolume(int32_t volume);
    int32_t getVolume();

    void getTrack(Track& track);
    void getAlbum(Album& album);
    bool getAlbumArt(uint32_t albumId, std::vector<uint8_t>& data);

    void quitApplication();
    void showHideWindow();

    bool getNextTrack(Track& track);

    Settings& getSettings();
    PluginManager& getPluginManager();
    PlayQueue& getPlayQueue();
    MusicLibrary& getMusicLibrary();
    IAlbumArtProvider& getAlbumArtProvider();

    void onTrackQueued(uint32_t index);

    Glib::Dispatcher dispatchProgress;
    Glib::Dispatcher dispatchVolumeChanged;
    Glib::Dispatcher dispatchNewTrackStarted;

private:
    void saveSettings();
    void loadSettings();

    enum FetchStatus
    {
        FetchStarted,
        FetchSuccess,
        FetchFailed
    };

    Glib::Dispatcher dispatchGetNextTrack;
    Glib::Dispatcher dispatchStop;
    void sendProgress();
    void sendVolumeChange();
    void newTrackStarted();
    void sendGetNextTrack();

    void addPlugin(GejengelPlugin& plugin);

    Settings            m_Settings;
    MusicLibrary        m_Library;
    PlayQueue           m_PlayQueue;
    PlaybackEngine*     m_pPlayback;
    PluginManager       m_PluginMgr;
    MainWindow          m_MainWindow;
    Track               m_CurrentTrack;
    uint32_t            m_CurrentProgress;
    FetchStatus         m_FetchStatus;

    friend class Dispatcher;
};

}

#endif
