//    Copyright (C) 2009 Dirk Vanden Boer <dirk.vdb@gmail.com>
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "musiclibrary.h"

#include <stdexcept>
#include <cassert>

#include "settings.h"
#include "MusicLibrary/scanner.h"
#include "Utils/log.h"
#include "Utils/trace.h"

using namespace std;

namespace Gejengel
{

MusicLibrary::MusicLibrary(const Settings& settings)
: m_Settings(settings)
, m_Db(settings.get("DBFile"), m_Dispatcher)
, m_ScannerThread(MusicLibrary::scannerThread, this)
{
    Utils::trace("Create MusicLibrary");
}

MusicLibrary::~MusicLibrary()
{
}

uint32_t MusicLibrary::getTrackCount()
{
    return m_Db.getTrackCount();
}

uint32_t MusicLibrary::getAlbumCount()
{
    return m_Db.getAlbumCount();
}

void MusicLibrary::getTrack(uint32_t id, Track& track)
{
    m_Db.getTrack(id, track);
}

void MusicLibrary::getRandomTracks(uint32_t trackCount, ITrackSubscriber& subscriber)
{
    m_Db.getRandomTracks(trackCount, subscriber);
}

void MusicLibrary::getRandomAlbum(ITrackSubscriber& subscriber)
{
    m_Db.getRandomAlbum(subscriber);
}

void MusicLibrary::getTracksFromAlbum(uint32_t albumId, ITrackSubscriber& subscriber)
{
    m_Db.getTracksFromAlbum(albumId, subscriber);
}

void MusicLibrary::getAlbums(IAlbumSubscriber& subscriber)
{
    m_Db.getAlbums(subscriber);
}

void MusicLibrary::getAlbum(uint32_t albumId, Album& album)
{
    m_Db.getAlbum(albumId, album);
}

bool MusicLibrary::getAlbumArt(uint32_t albumId, std::vector<uint8_t>& data)
{
    return m_Db.getAlbumArt(albumId, data);
}

void MusicLibrary::scan(bool startFresh)
{
    string libraryPath = m_Settings.get("MusicLibrary");

    if (startFresh || (m_LibraryPath != libraryPath && !m_LibraryPath.empty()))
    {
        m_Db.clearDatabase();
    }

    m_LibraryPath = libraryPath;
    m_ScannerThread.start();
}

void MusicLibrary::addLibrarySubscriber(ILibrarySubscriber& subscriber)
{
    m_Dispatcher.addSubscriber(subscriber);
}

void MusicLibrary::search(const std::string& searchString, ITrackSubscriber& trackSubscriber, IAlbumSubscriber& albumSubscriber)
{
    m_Db.searchLibrary(searchString, trackSubscriber, albumSubscriber);
}

void* MusicLibrary::scannerThread(void* pInstance)
{
    MusicLibrary* pLib = reinterpret_cast<MusicLibrary*>(pInstance);

    try
    {
        Scanner scanner(pLib->m_Db, pLib->m_Dispatcher, pLib->m_Settings.get("AlbumArtFilenames"));
        scanner.performScan(pLib->m_LibraryPath);
        pLib->m_Db.removeNonExistingFiles();
        pLib->m_Db.removeNonExistingAlbums();
    }
    catch (exception& e)
    {
        Log::error(string("Failed to scan library: ") + e.what());
        pLib->m_Dispatcher.scanFailed();
    }

    return NULL;
}

}
