//    Copyright (C) 2009 Dirk Vanden Boer <dirk.vdb@gmail.com>
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef MUSIC_LIBRARY_H
#define MUSIC_LIBRARY_H

#include <string>
#include <vector>

#include "Utils/types.h"
#include "Utils/thread.h"
#include "MusicLibrary/musicdb.h"
#include "MusicLibrary/subscribers.h"
#include "MusicLibrary/librarychangedispatcher.h"

namespace Gejengel
{

struct Track;
struct Album;
class Settings;

class MusicLibrary
{
public:
    MusicLibrary(const Settings& settings);
    ~MusicLibrary();

    uint32_t getTrackCount();
    uint32_t getAlbumCount();
    void getTrack(uint32_t id, Track& track);
    void getRandomTracks(uint32_t trackCount, ITrackSubscriber& subscriber);
    void getRandomAlbum(ITrackSubscriber& subscriber);
    void getTracksFromAlbum(uint32_t albumId, ITrackSubscriber& subscriber);
    void getAlbums(IAlbumSubscriber& subscriber);
    void getAlbum(uint32_t albumId, Album& album);
    bool getAlbumArt(uint32_t albumId, std::vector<uint8_t>& data);

    void scan(bool startFresh);
    void addLibrarySubscriber(ILibrarySubscriber& subscriber);
    void search(const std::string& search, ITrackSubscriber& trackSubscriber, IAlbumSubscriber& albumSubscriber);

private:
    static void* scannerThread(void* pInstance);

    void sendNewTrack(const Track& track);
    void sendNewAlbum(const Album& album);
    void sendDeletedTrack(const Track& track);
    void sendDeletedAlbum(const Album& album);

    const Settings&                 m_Settings;
    LibraryChangeDispatcher         m_Dispatcher;
    MusicDb                         m_Db;
    std::string                     m_LibraryPath;

    utils::Thread                   m_ScannerThread;
};

}

#endif //MUSIC_LIBRARY_H
