//    Copyright (C) 2009 Dirk Vanden Boer <dirk.vdb@gmail.com>
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef PLAYBACK_H
#define PLAYBACK_H

#include <fstream>
#include <vector>
#include <sigc++/connection.h>

#include "playbackengine.h"
#include "AudioRenderer/audioframe.h"
#include "MusicLibrary/track.h"
#include "Utils/types.h"
#include "Utils/mutex.h"
#include "Utils/condition.h"
#include "Utils/thread.h"


//#define DUMP_TO_WAVE

namespace Gejengel
{

class AudioDecoder;
class AudioRenderer;
class GejengelCore;

class Playback : public PlaybackEngine
{
public:
    Playback(GejengelCore& core);
    virtual ~Playback();

    void play();
    void pause();
    void resume();
    void stop();
    void prev();
    void next();
    bool isPaused();
    bool isPlaying();

    void seek(double seconds);
    double getCurrentTime();
    double getDuration();
    PlaybackState getState();

    void setVolume(int32_t volume);
    int32_t getVolume() const;
    const Track& getTrack() const;

private:
    void stopPlayback(bool drain);
    bool startNewTrack();
    void sendProgressIfNeeded();
    void playback();
    void playbackLoop();
    static void* playbackThread(void* pInstance);
    bool rendererHasSpace();

    AudioDecoder*       m_pAudioDecoder;
    AudioRenderer*      m_pAudioRenderer;

    utils::Thread       m_PlaybackThread;
    utils::Condition    m_PlaybackCondition;
    utils::Mutex        m_PlaybackMutex;
    utils::Mutex        m_DecodeMutex;

    Track               m_CurrentTrack;
    bool                m_Destroy;
    bool                m_Stop;
    bool                m_NewTrackStarted;
    PlaybackState       m_State;
    bool                m_SkipTrack;
    bool                m_SeekOccured;
    double              m_CurrentPts;

#ifdef DUMP_TO_WAVE
    std::ofstream       m_WaveFile;
    uint32_t            m_WaveBytes;

    void writeWaveHeader();
    void updateWaveHeaderSize();
    void dumpToWav(AudioFrame& frame);
#endif
};

}

#endif
