//    Copyright (C) 2009 Dirk Vanden Boer <dirk.vdb@gmail.com>
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "playbackenginefactory.h"

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef HAVE_GSTREAMER
#include "gstreamerengine.h"
#endif

#include "playback.h"
#include <stdexcept>

namespace Gejengel
{

PlaybackEngine* PlaybackEngineFactory::create(const std::string& engine, GejengelCore& core)
{
    if (engine == "GStreamer")
    {
#ifdef HAVE_GSTREAMER
        return new GStreamerEngine(core);
#else
        throw std::logic_error("PlaybackEngineFactory: package was not comiled with Gstreamer support");
#endif
    }

    if (engine == "FFmpeg")
    {
        return new Playback(core);
    }

    throw std::logic_error("PlaybackEngineFactory: Unsupported playback engine type provided: " + engine);
}

}
