//    Copyright (C) 2009 Dirk Vanden Boer <dirk.vdb@gmail.com>
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef PLAYQUEUE_H
#define PLAYQUEUE_H

#include "MusicLibrary/track.h"
#include "MusicLibrary/subscribers.h"
#include "Utils/types.h"

#include <list>
#include <vector>

namespace Gejengel
{

class MusicLibrary;

class PlayQueueSubscriber
{
public:
    virtual ~PlayQueueSubscriber() {};

    virtual void onTrackQueued(uint32_t index) {}
    virtual void onTrackRemoved(uint32_t index) {}
    virtual void onTrackMoved(uint32_t sourceIndex, uint32_t destIndex) {}
    virtual void onQueueCleared() {}
};

class PlayQueue
{
public:
    PlayQueue(MusicLibrary& library);
    ~PlayQueue();

    void queueTrack(const Track& track, int32_t index = -1);
    void queueTrack(uint32_t id, int32_t index = -1);
    void queueAlbum(uint32_t id, int32_t index = -1);

    void queueRandomTracks(uint32_t count);
    void queueRandomAlbum();

    bool getNextTrack(Track& track);
    void removeTrack(uint32_t index);
    void removeTracks(std::vector<uint32_t> indexes);
    void moveTrack(uint32_t sourceIndex, uint32_t destIndex);
    bool getTrackInfo(uint32_t index, Track& track);
    void clear();
    uint32_t size() const;
    const std::list<Track>& getTracks();

    void subscribe(PlayQueueSubscriber& subscriber);
    void unsubscribe(PlayQueueSubscriber& subscriber);

    void loadQueue();
    void saveQueue();

private:
    void notifyTrackQueued(uint32_t index);
    void notifyTrackRemoved(uint32_t index);
    void notifyTrackMoved(uint32_t sourceIndex, uint32_t destIndex);
    void notifyQueueCleared();

    static std::string determinePlayQueuePath();

    MusicLibrary&                       m_Library;
    std::list<Track>                    m_Tracks;
    std::vector<PlayQueueSubscriber*>   m_Subscribers;
};

}

#endif
