//    Copyright (C) 2009 Dirk Vanden Boer <dirk.vdb@gmail.com>
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef LIBRARY_CHANGE_DISPATCHER
#define LIBRARY_CHANGE_DISPATCHER

#include <glibmm/dispatcher.h>

#include "Utils/types.h"
#include "Utils/mutex.h"
#include "subscribers.h"
#include "track.h"
#include "album.h"

namespace Gejengel
{

class LibraryChangeDispatcher : public ILibrarySubscriber
{
public:
    LibraryChangeDispatcher();

    void addSubscriber(ILibrarySubscriber& subscriber);

    void newTrack(const Track& track);
    void newAlbum(const Album& album);
    void deletedTrack(uint32_t trackId);
    void deletedAlbum(uint32_t albumId);
    void updatedTrack(const Track& track);
    void updatedAlbum(const Album& album);
    void scanStart(uint32_t numTracks);
    void scanUpdate(uint32_t scannedTracks);
    void scanFinish();
    void scanFailed();
    void libraryCleared();

private:
    Glib::Dispatcher sendTrackAdded;
    Glib::Dispatcher sendTrackDeleted;
    Glib::Dispatcher sendTrackUpdated;
    Glib::Dispatcher sendAlbumAdded;
    Glib::Dispatcher sendAlbumDeleted;
    Glib::Dispatcher sendAlbumUpdated;
    Glib::Dispatcher sendScanStart;
    Glib::Dispatcher sendScanUpdate;
    Glib::Dispatcher sendScanFinish;
    Glib::Dispatcher sendScanFailed;
    Glib::Dispatcher sendLibraryCleared;

    void dispatchNewTrack();
    void dispatchNewAlbum();
    void dispatchDeletedTrack();
    void dispatchDeletedAlbum();
    void dispatchUpdatedTrack();
    void dispatchUpdatedAlbum();
    void dispatchScanStart();
    void dispatchScanUpdate();
    void dispatchScanFinish();
    void dispatchScanFailed();
    void dispatchLibraryCleared();

    std::vector<ILibrarySubscriber*>    m_Subscribers;
    std::vector<Track>                  m_NewTracks;
    std::vector<Album>                  m_NewAlbums;
    std::vector<uint32_t>               m_DeletedTracks;
    std::vector<uint32_t>               m_DeletedAlbums;
    std::vector<Track>                  m_UpdatedTracks;
    std::vector<Album>                  m_UpdatedAlbums;
    uint32_t                            m_NumTracks;
    uint32_t                            m_ScannedTracks;
    utils::Mutex                        m_VectorMutex;
};

}

#endif
