//    Copyright (C) 2009 Dirk Vanden Boer <dirk.vdb@gmail.com>
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <csignal>
#include <cerrno>
#include <string>
#include <cstring>
#include <gtkmm/main.h>
#include <libintl.h>

#ifdef HAVE_CONFIG_H
    #include "config.h"
#endif

#ifdef HAVE_GSTREAMER
    #include <gstreamermm/init.h>
#endif

#include "Core/gejengel.h"
#include "Utils/log.h"
#include "Utils/trace.h"

static bool set_signal_handlers();

int main(int argc, char **argv)
{
    Utils::trace("Application start");
    
#ifndef WIN32
    if (!set_signal_handlers())
    {
        return -1;
    }

    bindtextdomain(GETTEXT_PACKAGE, GEJENGEL_LOCALEDIR);
    bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
    textdomain(GETTEXT_PACKAGE);
#else
    Glib::thread_init();
#endif

#ifdef HAVE_GSTREAMER
    Gst::init(argc, argv);
#endif

    Gtk::Main main(&argc, &argv);

    Utils::trace("Create Gejengel core");
    Gejengel::GejengelCore core;
    
    Utils::trace("Run gtk main");
    core.run();

    return 0;
}

static void sigterm(int signo)
{
    Gtk::Main::quit();
}

#ifndef WIN32
static bool set_signal_handlers()
{
    struct sigaction sa;

    sa.sa_flags = 0;
    sa.sa_handler = sigterm;
    sigemptyset(&sa.sa_mask);
    sigaddset(&sa.sa_mask, SIGQUIT);
    sigaddset(&sa.sa_mask, SIGTERM);

    if (sigaction(SIGINT, &sa, NULL) < 0)
    {
        Log::error("Can't catch SIGINT:", strerror(errno));
        return false;
    }

    sa.sa_flags = 0;
    sa.sa_handler = sigterm;
    sigemptyset(&sa.sa_mask);
    sigaddset(&sa.sa_mask, SIGINT);
    sigaddset(&sa.sa_mask, SIGTERM);

    if (sigaction(SIGQUIT, &sa, NULL) < 0)
    {
        Log::error("Can't catch SIGQUIT:", strerror(errno));
        return false;
    }

    sa.sa_handler = sigterm;
    sigemptyset(&sa.sa_mask);
    sigaddset(&sa.sa_mask, SIGINT);
    sigaddset(&sa.sa_mask, SIGQUIT);
    sa.sa_flags = 0;

    if (sigaction(SIGTERM, &sa, NULL) < 0)
    {
        Log::error("Can't catch SIGTERM:", strerror(errno));
        return false;
    }

    return true;
}

#endif
