//    Copyright (C) 2009 Dirk Vanden Boer <dirk.vdb@gmail.com>
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef ALBUM_MODEL_H
#define ALBUM_MODEL_H

#include <gtkmm.h>

#include "Utils/types.h"
#include "MusicLibrary/subscribers.h"

namespace Gejengel
{

struct Album;
class IAlbumArtProvider;

class AlbumModel : public IAlbumSubscriber
{
public:
    static const uint32_t AlbumArtNeedsUpdate;

    class Columns : public Gtk::TreeModel::ColumnRecord
    {
    public:
        Columns()
        {
            add(albumArt); add(title); add(artist);
            add(duration); add(year); add(id); add(albumArtSize); add(dateAdded);
        }

        Gtk::TreeModelColumn<uint32_t>                      id;
        Gtk::TreeModelColumn<Glib::RefPtr<Gdk::Pixbuf> >    albumArt;
        Gtk::TreeModelColumn<Glib::ustring>                 title;
        Gtk::TreeModelColumn<Glib::ustring>                 artist;
        Gtk::TreeModelColumn<Glib::ustring>                 duration;
        Gtk::TreeModelColumn<uint32_t>                      year;
        Gtk::TreeModelColumn<uint32_t>                      dateAdded;
        Gtk::TreeModelColumn<uint32_t>                      albumArtSize;
    };

    AlbumModel(IAlbumArtProvider& artProvider);
    virtual ~AlbumModel();

    void addAlbum(const Album& album);
    void deleteAlbum(uint32_t id);
    void updateAlbum(const Album& album);
    bool fetchAlbumArt(uint32_t albumId, int32_t size);
    void clearAlbumArtCache();
    const Columns& columns();
    void setSortColumn(int32_t id, Gtk::SortType type);
    void getSortColumn(int32_t& id, Gtk::SortType& type);
    void clear();

    /* IAlbumSubscriber interface */
    void onAlbum(const Album& album);

    Glib::RefPtr<Gtk::ListStore> getStore() { return m_ListStore; }

private:
    IAlbumArtProvider&              m_AlbumArtProvider;
    Glib::RefPtr<Gtk::ListStore>    m_ListStore;
    Columns                         m_Columns;
};

}

#endif /* ALBUM_MODEL_H */
