//    Copyright (C) 2009 Dirk Vanden Boer <dirk.vdb@gmail.com>
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef CELL_RENDERER_ALBUM_H
#define CELL_RENDERER_ALBUM_H

#include <gdkmm.h>
#include <gtkmm.h>

#include "Utils/types.h"

namespace Gejengel
{

class IAlbumArtProvider;
class AlbumModel;
struct MouseInfo;

class CellRendererAlbum : public Gtk::CellRenderer
{
public:
    enum Size
    {
        Large,
        Small
    };

    CellRendererAlbum(AlbumModel& model, MouseInfo& mousePos);
    ~CellRendererAlbum();

    void setSize(Size size);
    Size getSize();

    Glib::PropertyProxy<uint32_t>                   property_id();
    Glib::PropertyProxy<uint32_t>                   property_album_art_size();
    Glib::PropertyProxy<Glib::RefPtr<Gdk::Pixbuf> > property_album_art();
    Glib::PropertyProxy<Glib::ustring>              property_title();
    Glib::PropertyProxy<Glib::ustring>              property_artist();
    Glib::PropertyProxy<uint32_t>                   property_year();
    Glib::PropertyProxy<Glib::ustring>              property_duration();

    void get_size_vfunc(Gtk::Widget& widget, const Gdk::Rectangle* cellArea,
                        int32_t* x_offset, int32_t* y_offset,
                        int32_t* width, int32_t* height) const;
    void render_vfunc(  const Glib::RefPtr<Gdk::Drawable>& window, Gtk::Widget& widget,
                        const Gdk::Rectangle& bgArea, const Gdk::Rectangle& cellArea,
                        const Gdk::Rectangle& exposeArea, Gtk::CellRendererState flags);

    void setButtonStyle(Glib::RefPtr<Gtk::Style> style);

private:
    void iconThemeChanged();
    void loadIcons();
    Glib::RefPtr<Pango::Layout> createLayout(Gtk::Widget& widget) const;

    Glib::Property<uint32_t>                    m_IdProperty;
    Glib::Property<uint32_t>                    m_AlbumArtSizeProperty;
    Glib::Property<Glib::RefPtr<Gdk::Pixbuf> >  m_AlbumArtProperty;
    Glib::Property<Glib::ustring>               m_TitleProperty;
    Glib::Property<Glib::ustring>               m_ArtistProperty;
    Glib::Property<uint32_t>                    m_YearProperty;
    Glib::Property<Glib::ustring>               m_DurationProperty;
    Glib::RefPtr<Gdk::Pixbuf>                   m_DefaultAlbumArt;
    Glib::RefPtr<Gdk::Pixbuf>                   m_PlayImage;
    Glib::RefPtr<Gtk::Style>                    m_ButtonStyle;
    AlbumModel&                                 m_AlbumModel;
    MouseInfo&                                  m_MouseInfo;
    Size                                        m_Size;
    int32_t                                     m_IconSize;
    int32_t                                     m_HoverSize;
    int32_t                                     m_OverlaySize;
    int32_t                                     m_HoverOffset;
    int32_t                                     m_OverlayOffset;
};

}

#endif
