//    Copyright (C) 2009 Dirk Vanden Boer <dirk.vdb@gmail.com>
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef MAIN_WINDOW_H
#define MAIN_WINDOW_H

#include <gtkmm.h>

#include "trackmodel.h"
#include "albummodel.h"
#include "playqueuemodel.h"
#include "uilayoutfactory.h"
#include "systemtray.h"
#include "Utils/types.h"

namespace Gejengel
{

struct Track;
struct Album;
class IGejengelCore;
class GejengelPlugin;
class UILayout;
class IAlbumArtProvider;

class MainWindow    : public Gtk::Window
                    , public ILibrarySubscriber
{
public:
    MainWindow(IGejengelCore& core);
    ~MainWindow();

    void selectedAlbumChanged(uint32_t albumId);
    void showHideWindow();
    void run();
    void quit();
    void pushStatusMessage(const Glib::ustring& message);

    /* LibrarySubscriber interface */
    void newTrack(const Track& track);
    void newAlbum(const Album& album);
    void deletedTrack(uint32_t id);
    void deletedAlbum(uint32_t id);
    void updatedAlbum(const Album& album);
    void scanStart(uint32_t numTracks);
    void scanUpdate(uint32_t scannedTracks);
    void scanFinish();
    void scanFailed();
    void libraryCleared();

protected:
    void saveSettings();
    void loadSettings();
    bool on_delete_event(GdkEventAny* pEvent);

    void init();

    void showPreferences();
    void showAbout();
    void scanLibrary(bool freshStart);
    void onScanLibrary();
    void onFullscreen();
    void onQueueRandomTracks();
    void onQueueRandomAlbum();
    void onSearchChanged(const Glib::ustring& search);
    void onLayoutChange();
    bool onKeyPress(GdkEventKey* event);
    bool onWindowStateChanged(GdkEventWindowState* pState);
    void loadLayout();

    IGejengelCore&                  m_Core;
    Glib::RefPtr<Gtk::UIManager>    m_UIManager;
    Glib::RefPtr<Gtk::ActionGroup>  m_ActionGroup;
    Gtk::VBox                       m_EntireLayout;
    Gtk::VBox                       m_InfoLayout;
    Gtk::Statusbar                  m_StatusBar;
    Gtk::ProgressBar                m_ScanProgress;
    TrackModel                      m_TrackModel;
    AlbumModel                      m_AlbumModel;
    PlayQueueModel                  m_PlayQueueModel;
    UILayoutFactory::LayoutStyle    m_LayoutStyle;
    UILayout*                       m_pLayout;
    uint32_t                        m_TracksToScan;
    SystemTray                      m_SystemTray;
    bool                            m_CloseToTray;
};

}

#endif
