//    Copyright (C) 2009 Dirk Vanden Boer <dirk.vdb@gmail.com>
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "sharedfunctions.h"

#include <sstream>
#include <iomanip>

#include "overlay.h"
#include "Utils/stringoperations.h"
#include "MusicLibrary/album.h"

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <Magick++.h>

using namespace std;

namespace Gejengel
{

namespace Shared
{

void durationToString(uint32_t duration, Glib::ustring& durationString)
{
    uint32_t hours   = duration / 3600;
    duration -= hours * 3600;

    uint32_t minutes = duration / 60;
    uint32_t seconds = duration % 60;


    stringstream ss;

    if (hours > 0)
    {
        ss << setfill('0') << hours << ':';
    }
    ss << setw(2) << setfill('0') << minutes << ':' << setw(2) << setfill('0')  << seconds;
    durationString = ss.str();
}

uint32_t durationFromString(const Glib::ustring& durationString)
{
    uint32_t duration = 0;
    std::vector<std::string> times = StringOperations::tokenize(durationString, ":");

    if (times.size() == 3)
    {
        uint32_t hours, minutes, seconds;
        StringOperations::toNumeric(times[0], hours);
        StringOperations::toNumeric(times[1], minutes);
        StringOperations::toNumeric(times[2], seconds);

        duration += hours * 3600;
        duration += minutes * 60;
        duration += seconds;
    }
    else if (times.size() == 3)
    {
        uint32_t minutes, seconds;
        StringOperations::toNumeric(times[0], minutes);
        StringOperations::toNumeric(times[1], seconds);

        duration += minutes * 60;
        duration += seconds;
    }

    return duration;
}

Glib::RefPtr<Gdk::Pixbuf> createCoverPixBuf(const Album& album, int32_t size)
{
    Glib::RefPtr<Gdk::PixbufLoader> loader = Gdk::PixbufLoader::create();
    loader->set_size(size, size);
    loader->write(&album.coverData.front(), album.coverData.size());
    Glib::RefPtr<Gdk::Pixbuf> pixBuf = loader->get_pixbuf();
    loader->close();

    return pixBuf;
}

Glib::RefPtr<Gdk::Pixbuf> createCoverPixBufWithOverlay(const Album& album, int32_t size)
{
    try
    {
        Magick::Blob coverImageBlob(&album.coverData.front(), album.coverData.size());
        Magick::Image coverImage(coverImageBlob);

        std::stringstream finalGeometry;
        finalGeometry << size << "x" << size;

        std::stringstream overlayGeometry;
        overlayGeometry << coverWidth << "x" << coverHeight << "!";

        Magick::Blob cdCaseBlob(cdCaseData, sizeof(cdCaseData));
        Magick::Image cdCase(cdCaseBlob);

        coverImage.resize(Magick::Geometry(overlayGeometry.str()));
        cdCase.composite(coverImage, coverImageOffsetX, coverImageOffsetY, Magick::PlusCompositeOp);
        cdCase.resize(Magick::Geometry(finalGeometry.str()));

        Magick::Blob data;
        cdCase.write(&data, "PNG");

        Glib::RefPtr<Gdk::PixbufLoader> loader = Gdk::PixbufLoader::create();
        loader->set_size(size, size);
        loader->write((guint8*)data.data(), data.length());
        Glib::RefPtr<Gdk::Pixbuf> pixBuf = loader->get_pixbuf();
        loader->close();

        return pixBuf;
    }
    catch (Magick::Exception&)
    {
        return createCoverPixBuf(album, size);
    }
}

}

}
