//    Copyright (C) 2009 Dirk Vanden Boer <dirk.vdb@gmail.com>
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "simplealbumlayout.h"

#include "mainwindow.h"
#include "Core/settings.h"
#include "Utils/log.h"
#include "Utils/trace.h"

namespace Gejengel
{

SimpleAlbumLayout::SimpleAlbumLayout(Settings& settings, PlayQueueModel& playQueueModel, AlbumModel& albumModel)
: m_Settings(settings)
, m_AlbumView(settings, albumModel)
, m_PlayQueueView(playQueueModel)
, m_NowPlaying()
{
    Utils::trace("Create detailed album layout");
    m_AlbumQueuePane.pack1(m_AlbumView, Gtk::SHRINK);
    m_AlbumQueuePane.pack2(m_PlayQueueView, Gtk::SHRINK);

    m_Layout.pack_start(m_NowPlaying.getWidget(), Gtk::PACK_SHRINK);
    m_Layout.pack_start(m_AlbumQueuePane, Gtk::PACK_EXPAND_WIDGET);
    m_Layout.set_spacing(5);

    m_AlbumView.signalAlbumQueued.connect(sigc::mem_fun(playQueueModel.getQueue(), &PlayQueue::queueAlbum));
    m_NowPlaying.signalSearchChanged.connect(sigc::mem_fun(signalSearchChanged, &sigc::signal<void, const Glib::ustring&>::emit));

    loadSettings();
}

SimpleAlbumLayout::~SimpleAlbumLayout()
{
    saveSettings();
}

void SimpleAlbumLayout::loadSettings()
{
    m_AlbumQueuePane.set_position(m_Settings.getAsInt("SimpleAlbumLayoutDividerPos", 250));
}

void SimpleAlbumLayout::saveSettings()
{
    m_Settings.set("SimpleAlbumLayoutDividerPos", m_AlbumQueuePane.get_position());
}


Gtk::Widget& SimpleAlbumLayout::getWidget()
{
    return m_Layout;
}

void SimpleAlbumLayout::getPlugins(std::vector<GejengelPlugin*>& plugins)
{
    plugins.push_back(&m_NowPlaying);
}

void SimpleAlbumLayout::focusSearchField()
{
    m_NowPlaying.focusSearchField();
}

}
